@@ -41,6 +41,7 @@ ENFORCEMENT_POINT_PATTERN = (TENANTS_PATH_PATTERN +
                              "sites/default/enforcement-points/")
 TRANSPORT_ZONE_PATTERN = ENFORCEMENT_POINT_PATTERN + "%s/transport-zones/"
 EDGE_CLUSTER_PATTERN = ENFORCEMENT_POINT_PATTERN + "%s/edge-clusters/"
+AVI_ENDPOINT_PATTERN = ENFORCEMENT_POINT_PATTERN + "alb-endpoint"
 
 SEGMENT_SECURITY_PROFILES_PATH_PATTERN = (TENANTS_PATH_PATTERN +
                                           "segment-security-profiles/")
@@ -1166,10 +1167,11 @@ class SegmentPortDef(ResourceDef):
                 body['address_bindings'] = [binding.get_obj_dict()
                                             for binding in address_bindings]
         if (self.has_attr('attachment_type') or self.has_attr('vif_id') or
-            self.has_attr('hyperbus_mode')):
+            self.has_attr('hyperbus_mode') or self.has_attr('app_id')):
             if (not self.get_attr('attachment_type') and
                 not self.get_attr('vif_id') and
-                not self.get_attr('hyperbus_mode')):
+                not self.get_attr('hyperbus_mode') and
+                not self.get_attr('app_id')):
                 # detach operation
                 body['attachment'] = None
             else:
@@ -1609,18 +1611,35 @@ class IpPoolStaticSubnetDef(IpPoolSubnetDef):
 class Condition(object):
     def __init__(self, value, key=constants.CONDITION_KEY_TAG,
                  member_type=constants.CONDITION_MEMBER_PORT,
-                 operator=constants.CONDITION_OP_EQUALS):
+                 operator=constants.CONDITION_OP_EQUALS,
+                 scope_operator=None, nsx_version=None):
+        # Adding NSX version for 3.2.0, new field 'scope_operator' added
         self.value = value
         self.key = key
         self.member_type = member_type
         self.operator = operator
+        self.scope_operator = scope_operator
+        self.nsx_version = nsx_version
 
     def get_obj_dict(self):
-        return {'resource_type': 'Condition',
+        body = {'resource_type': 'Condition',
                 'member_type': self.member_type,
                 'key': self.key,
                 'value': self.value,
                 'operator': self.operator}
+        if self.scope_operator is not None:
+            if (version.LooseVersion(self.nsx_version) >=
+                    version.LooseVersion(nsx_constants.NSX_VERSION_3_2_0)):
+                if self.scope_operator == 'NOTEQUALS':
+                    del body['operator']
+                body['scope_operator'] = self.scope_operator
+            else:
+                LOG.warning(
+                    "Ignoring scope_operator '%s' for NSX Policy Group : this "
+                    "feature is not supported.Current NSX version: %s. Minimum"
+                    " supported version: %s", self.scope_operator,
+                    self.nsx_version, '3.2.0')
+        return body
 
     def __eq__(self, other):
         if isinstance(other, Condition):
@@ -2550,11 +2569,14 @@ class NsxPolicyApi(object):
 
         self.client.patch(path, body)
 
-    def delete(self, resource_def):
+    def delete(self, resource_def, force=False):
+        headers = None
+        if force:
+            headers = {'X-Allow-Overwrite': 'true'}
         path = resource_def.get_resource_path()
         if resource_def.resource_use_cache():
             self.cache.remove(path)
-        self.client.delete(path)
+        self.client.delete(path, headers=headers)
 
     def get(self, resource_def, silent=False):
         path = resource_def.get_resource_path()
